@@ -36,7 +36,7 @@ GST_ELEMENT_DETAILS ("JPEG image decoder",
 
 #define MIN_WIDTH  16
 #define MAX_WIDTH  4096
-#define MIN_HEIGHT 16
+#define MIN_HEIGHT 8
 #define MAX_HEIGHT 4096
 
 #define DEFAULT_IDCT_METHOD	JDCT_FASTEST
@@ -467,7 +467,7 @@ gst_jpeg_dec_parse_image_data (GstJpegDec * dec)
         ++data;
         continue;
       }
-      GST_DEBUG ("found sync at 0x%08x", data - size);
+      GST_DEBUG ("found sync at %p", data - size);
     }
     while (*data == 0xff)
       ++data;
@@ -1072,21 +1072,20 @@ gst_jpeg_dec_sink_event (GstPad * pad, GstEvent * event)
       break;
     case GST_EVENT_NEWSEGMENT:{
       gboolean update;
-      gdouble rate;
+      gdouble rate, applied_rate;
       GstFormat format;
       gint64 start, stop, position;
 
-      /* Once -good depends on core >= 0.10.6, use newsegment_full */
-      gst_event_parse_new_segment (event, &update, &rate, &format,
-          &start, &stop, &position);
+      gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
+          &format, &start, &stop, &position);
 
       GST_DEBUG_OBJECT (dec, "Got NEWSEGMENT [%" GST_TIME_FORMAT
           " - %" GST_TIME_FORMAT " / %" GST_TIME_FORMAT "]",
           GST_TIME_ARGS (start), GST_TIME_ARGS (stop),
           GST_TIME_ARGS (position));
 
-      gst_segment_set_newsegment (dec->segment, update, rate, format,
-          start, stop, position);
+      gst_segment_set_newsegment_full (dec->segment, update, rate,
+          applied_rate, format, start, stop, position);
 
       break;
     }
