@@ -329,6 +329,7 @@ gst_tag_demux_add_srcpad (GstTagDemux * tagdemux, GstCaps * new_caps)
         tagdemux->priv->src_caps);
 
     gst_object_ref (tagdemux->priv->srcpad);
+    gst_pad_set_active (tagdemux->priv->srcpad, TRUE);
     if (!(gst_element_add_pad (GST_ELEMENT (tagdemux), tagdemux->priv->srcpad)))
       return FALSE;
     gst_element_no_more_pads (GST_ELEMENT (tagdemux));
@@ -457,11 +458,13 @@ gst_tag_demux_chain_parse_tag (GstTagDemux * demux, GstBuffer * collect)
   guint tagsize = 0;
   guint available;
 
+  g_assert (gst_buffer_is_metadata_writable (collect));
+
   klass = GST_TAG_DEMUX_CLASS (G_OBJECT_GET_CLASS (demux));
 
   /* If we receive a buffer that's from the middle of the file, 
    * we can't read tags so move to typefinding */
-  if (GST_BUFFER_OFFSET (collect) != 0) {
+  if (GST_BUFFER_OFFSET_IS_VALID (collect) && GST_BUFFER_OFFSET (collect) != 0) {
     GST_DEBUG_OBJECT (demux, "Received buffer from non-zero offset %"
         G_GINT64_FORMAT ". Can't read tags", GST_BUFFER_OFFSET (collect));
     demux->priv->state = GST_TAG_DEMUX_TYPEFINDING;
@@ -485,6 +488,12 @@ gst_tag_demux_chain_parse_tag (GstTagDemux * demux, GstBuffer * collect)
     return;
   }
 
+  /* need to set offset of first buffer to 0 or trimming won't work */
+  if (!GST_BUFFER_OFFSET_IS_VALID (collect)) {
+    GST_WARNING_OBJECT (demux, "Fixing up first buffer without offset");
+    GST_BUFFER_OFFSET (collect) = 0;
+  }
+
   GST_DEBUG_OBJECT (demux, "Identified tag, size = %u bytes", tagsize);
 
   do {
@@ -549,6 +558,8 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
 
   switch (demux->priv->state) {
     case GST_TAG_DEMUX_READ_START_TAG:
+      demux->priv->collect =
+          gst_buffer_make_metadata_writable (demux->priv->collect);
       gst_tag_demux_chain_parse_tag (demux, demux->priv->collect);
       if (demux->priv->state != GST_TAG_DEMUX_TYPEFINDING)
         break;
