@@ -136,7 +136,7 @@ static GstStaticPadTemplate video_sink_factory =
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], "
         "framerate = (fraction) [ 0, MAX ], "
-        "mpegversion = (int) 1, "
+        "mpegversion = (int) { 1, 2, 4}, "
         "systemstream = (boolean) FALSE; "
         "video/x-h263, "
         "width = (int) [ 16, 4096 ], "
@@ -1109,7 +1109,7 @@ gst_avi_mux_write_avix_index (GstAviMux * avimux, guchar * code,
       /* msb is set if not (!) keyframe */
       GST_WRITE_UINT32_LE (buffdata + 4, GUINT32_FROM_LE (entry->size)
           | (GUINT32_FROM_LE (entry->flags)
-              & GST_RIFF_IF_KEYFRAME ? 0 : 1 << 31));
+              & GST_RIFF_IF_KEYFRAME ? 0 : 1U << 31));
       buffdata += 8;
     }
     i--;
@@ -1370,7 +1370,7 @@ gst_avi_mux_stop_file (GstAviMux * avimux)
   /* statistics/total_frames/... */
   avimux->avi_hdr.tot_frames = avimux->num_frames;
   if (avimux->video_pad_connected) {
-    avimux->vids_hdr.length = avimux->num_frames;
+    avimux->vids_hdr.length = avimux->total_frames;
   }
   if (avimux->audio_pad_connected) {
     avimux->auds_hdr.length =
