@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005,2006> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,6 +16,29 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/*
+ * Unless otherwise indicated, Source Code is licensed under MIT license.
+ * See further explanation attached in License Statement (distributed in the file
+ * LICENSE).
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of
+ * this software and associated documentation files (the "Software"), to deal in
+ * the Software without restriction, including without limitation the rights to
+ * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
+ * of the Software, and to permit persons to whom the Software is furnished to do
+ * so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
+ */
 /* Element-Checklist-Version: 5 */
 
 /**
@@ -30,7 +53,6 @@
  * Last reviewed on 2006-06-20 (0.10.4)
  */
 
-
 #include "gstrtpdec.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpdec_debug);
@@ -170,16 +192,16 @@ gst_rtpdec_init (GstRTPDec * rtpdec)
   rtpdec->sink_rtp =
       gst_pad_new_from_static_template (&gst_rtpdec_sink_rtp_template,
       "sinkrtp");
-  gst_element_add_pad (GST_ELEMENT (rtpdec), rtpdec->sink_rtp);
   gst_pad_set_getcaps_function (rtpdec->sink_rtp, gst_rtpdec_getcaps);
   gst_pad_set_chain_function (rtpdec->sink_rtp, gst_rtpdec_chain_rtp);
+  gst_element_add_pad (GST_ELEMENT (rtpdec), rtpdec->sink_rtp);
 
   /* the input rtcp pad */
   rtpdec->sink_rtcp =
       gst_pad_new_from_static_template (&gst_rtpdec_sink_rtcp_template,
       "sinkrtcp");
-  gst_element_add_pad (GST_ELEMENT (rtpdec), rtpdec->sink_rtcp);
   gst_pad_set_chain_function (rtpdec->sink_rtcp, gst_rtpdec_chain_rtcp);
+  gst_element_add_pad (GST_ELEMENT (rtpdec), rtpdec->sink_rtcp);
 
   /* the output rtp pad */
   rtpdec->src_rtp =
@@ -200,15 +222,28 @@ gst_rtpdec_getcaps (GstPad * pad)
   GstRTPDec *src;
   GstPad *other;
   GstCaps *caps;
+  const GstCaps *templ;
 
   src = GST_RTPDEC (GST_PAD_PARENT (pad));
 
-  other = pad == src->src_rtp ? src->sink_rtp : src->src_rtp;
+  other = (pad == src->src_rtp ? src->sink_rtp : src->src_rtp);
 
   caps = gst_pad_peer_get_caps (other);
 
-  if (caps == NULL)
-    caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  templ = gst_pad_get_pad_template_caps (pad);
+  if (caps == NULL) {
+    GST_DEBUG_OBJECT (src, "copy template");
+    caps = gst_caps_copy (templ);
+  } else {
+    GstCaps *intersect;
+
+    GST_DEBUG_OBJECT (src, "intersect with template");
+
+    intersect = gst_caps_intersect (caps, templ);
+    gst_caps_unref (caps);
+
+    caps = intersect;
+  }
 
   return caps;
 }
@@ -220,14 +255,18 @@ gst_rtpdec_chain_rtp (GstPad * pad, GstBuffer * buffer)
 
   src = GST_RTPDEC (GST_PAD_PARENT (pad));
 
-  GST_DEBUG ("got rtp packet");
+  GST_DEBUG_OBJECT (src, "got rtp packet");
   return gst_pad_push (src->src_rtp, buffer);
 }
 
 static GstFlowReturn
 gst_rtpdec_chain_rtcp (GstPad * pad, GstBuffer * buffer)
 {
-  GST_DEBUG ("got rtcp packet");
+  GstRTPDec *src;
+
+  src = GST_RTPDEC (GST_PAD_PARENT (pad));
+
+  GST_DEBUG_OBJECT (src, "got rtcp packet");
 
   gst_buffer_unref (buffer);
   return GST_FLOW_OK;
