@@ -52,6 +52,7 @@
 #include <gst/gst.h>
 #include <gst/base/gstcollectpads.h>
 
+#include <stdlib.h>
 #include <string.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_videomixer_debug);
@@ -64,7 +65,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_videomixer_debug);
         (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_MIXER_PAD, GstVideoMixerPadiClass))
 #define GST_IS_VIDEO_MIXER_PAD(obj) \
         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_MIXER_PAD))
-#define GST_IS_VIDEO_MIXER_PAD_CLASS(obj) \
+#define GST_IS_VIDEO_MIXER_PAD_CLASS(klass) \
         (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_MIXER_PAD))
 
 typedef struct _GstVideoMixerPad GstVideoMixerPad;
@@ -78,14 +79,76 @@ typedef struct _GstVideoMixerCollect GstVideoMixerCollect;
         (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_MIXER, GstVideoMixerClass))
 #define GST_IS_VIDEO_MIXER(obj) \
         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_MIXER))
-#define GST_IS_VIDEO_MIXER_CLASS(obj) \
+#define GST_IS_VIDEO_MIXER_CLASS(klass) \
         (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_MIXER))
 
+#define GST_VIDEO_MIXER_GET_STATE_LOCK(mix) \
+  (GST_VIDEO_MIXER(mix)->state_lock)
+#define GST_VIDEO_MIXER_STATE_LOCK(mix) \
+  (g_mutex_lock(GST_VIDEO_MIXER_GET_STATE_LOCK (mix)))
+#define GST_VIDEO_MIXER_STATE_UNLOCK(mix) \
+  (g_mutex_unlock(GST_VIDEO_MIXER_GET_STATE_LOCK (mix)))
+
 static GType gst_videomixer_get_type (void);
 
 typedef struct _GstVideoMixer GstVideoMixer;
 typedef struct _GstVideoMixerClass GstVideoMixerClass;
 
+/**
+ * GstVideoMixerBackground:
+ * @VIDEO_MIXER_BACKGROUND_CHECKER: checker pattern background
+ * @VIDEO_MIXER_BACKGROUND_BLACK: solid color black background
+ * @VIDEO_MIXER_BACKGROUND_WHITE: solid color white background
+ *
+ * The different backgrounds videomixer can blend over.
+ */
+typedef enum
+{
+  VIDEO_MIXER_BACKGROUND_CHECKER,
+  VIDEO_MIXER_BACKGROUND_BLACK,
+  VIDEO_MIXER_BACKGROUND_WHITE
+}
+GstVideoMixerBackground;
+
+/**
+ * GstVideoMixer:
+ *
+ * The opaque #GstVideoMixer structure.
+ */
+struct _GstVideoMixer
+{
+  GstElement element;
+
+  /* pad */
+  GstPad *srcpad;
+
+  /* Lock to prevent the state to change while blending */
+  GMutex *state_lock;
+  /* Sink pads using Collect Pads from core's base library */
+  GstCollectPads *collect;
+  /* sinkpads, a GSList of GstVideoMixerPads */
+  GSList *sinkpads;
+
+  gint numpads;
+
+  /* the master pad */
+  GstVideoMixerPad *master;
+
+  gint in_width, in_height;
+  gint out_width, out_height;
+  gboolean setcaps;
+
+  GstVideoMixerBackground background;
+
+  gint fps_n;
+  gint fps_d;
+};
+
+struct _GstVideoMixerClass
+{
+  GstElementClass parent_class;
+};
+
 static void gst_videomixer_pad_base_init (gpointer g_class);
 static void gst_videomixer_pad_class_init (GstVideoMixerPadClass * klass);
 static void gst_videomixer_pad_init (GstVideoMixerPad * mixerpad);
@@ -124,7 +187,7 @@ struct _GstVideoMixerPad
 {
   GstPad parent;                /* subclass the pad */
 
-  guint64 queued;
+  gint64 queued;
 
   guint in_width, in_height;
   gint fps_n;
@@ -235,8 +298,10 @@ gst_videomixer_pad_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_PAD_ZORDER:
+      GST_VIDEO_MIXER_STATE_LOCK (mix);
       pad->zorder = g_value_get_uint (value);
       gst_videomixer_sort_pads (mix);
+      GST_VIDEO_MIXER_STATE_UNLOCK (mix);
       break;
     case ARG_PAD_XPOS:
       pad->xpos = g_value_get_int (value);
@@ -255,57 +320,47 @@ gst_videomixer_pad_set_property (GObject * object, guint prop_id,
   gst_object_unref (mix);
 }
 
-/**
- * GstVideoMixerBackground:
- * @VIDEO_MIXER_BACKGROUND_CHECKER: checker pattern background
- * @VIDEO_MIXER_BACKGROUND_BLACK: solid color black background
- * @VIDEO_MIXER_BACKGROUND_WHITE: solid color white background
- *
- * The different backgrounds videomixer can blend over.
- */
-typedef enum
+static void
+gst_videomixer_set_master_geometry (GstVideoMixer * mix)
 {
-  VIDEO_MIXER_BACKGROUND_CHECKER,
-  VIDEO_MIXER_BACKGROUND_BLACK,
-  VIDEO_MIXER_BACKGROUND_WHITE
+  GSList *walk;
+  gint width = 0, height = 0, fps_n = 0, fps_d = 0;
+  GstVideoMixerPad *master = NULL;
+
+  walk = mix->sinkpads;
+  while (walk) {
+    GstVideoMixerPad *mixpad = GST_VIDEO_MIXER_PAD (walk->data);
+
+    walk = g_slist_next (walk);
+
+    /* Biggest input geometry will be our output geometry */
+    width = MAX (width, mixpad->in_width);
+    height = MAX (height, mixpad->in_height);
+
+    /* If mix framerate < mixpad framerate, using fractions */
+    GST_DEBUG_OBJECT (mix, "comparing framerate %d/%d to mixpad's %d/%d",
+        fps_n, fps_d, mixpad->fps_n, mixpad->fps_d);
+    if ((!fps_n && !fps_d) ||
+        ((gint64) fps_n * mixpad->fps_d < (gint64) mixpad->fps_n * fps_d)) {
+      fps_n = mixpad->fps_n;
+      fps_d = mixpad->fps_d;
+      GST_DEBUG_OBJECT (mixpad, "becomes the master pad");
+      master = mixpad;
+    }
+  }
+
+  /* set results */
+  if (mix->master != master || mix->in_width != width
+      || mix->in_height != height || mix->fps_n != fps_n
+      || mix->fps_d != fps_d) {
+    mix->setcaps = TRUE;
+    mix->master = master;
+    mix->in_width = width;
+    mix->in_height = height;
+    mix->fps_n = fps_n;
+    mix->fps_d = fps_d;
+  }
 }
-GstVideoMixerBackground;
-
-/**
- * GstVideoMixer:
- *
- * The opaque #GstVideoMixer structure.
- */
-struct _GstVideoMixer
-{
-  GstElement element;
-
-  /* pad */
-  GstPad *srcpad;
-
-  /* Sink pads using Collect Pads from core's base library */
-  GstCollectPads *collect;
-  /* sinkpads, a GSList of GstVideoMixerPads */
-  GSList *sinkpads;
-
-  gint numpads;
-
-  /* the master pad */
-  GstVideoMixerPad *master;
-
-  gint in_width, in_height;
-  gint out_width, out_height;
-
-  GstVideoMixerBackground background;
-
-  gint fps_n;
-  gint fps_d;
-};
-
-struct _GstVideoMixerClass
-{
-  GstElementClass parent_class;
-};
 
 static gboolean
 gst_videomixer_pad_sink_setcaps (GstPad * pad, GstCaps * vscaps)
@@ -320,9 +375,8 @@ gst_videomixer_pad_sink_setcaps (GstPad * pad, GstCaps * vscaps)
   mix = GST_VIDEO_MIXER (gst_pad_get_parent (pad));
   mixpad = GST_VIDEO_MIXER_PAD (pad);
 
-  if (!mixpad) {
+  if (!mixpad)
     goto beach;
-  }
 
   GST_DEBUG_OBJECT (mixpad, "setcaps triggered");
 
@@ -333,30 +387,15 @@ gst_videomixer_pad_sink_setcaps (GstPad * pad, GstCaps * vscaps)
       || (framerate = gst_structure_get_value (structure, "framerate")) == NULL)
     goto beach;
 
+  GST_VIDEO_MIXER_STATE_LOCK (mix);
   mixpad->fps_n = gst_value_get_fraction_numerator (framerate);
   mixpad->fps_d = gst_value_get_fraction_denominator (framerate);
 
   mixpad->in_width = in_width;
   mixpad->in_height = in_height;
 
-  mixpad->xpos = 0;
-  mixpad->ypos = 0;
-
-  /* Biggest input geometry will be our output geometry */
-  mix->in_width = MAX (mix->in_width, mixpad->in_width);
-  mix->in_height = MAX (mix->in_height, mixpad->in_height);
-
-  /* If mix framerate < mixpad framerate, using fractions */
-  GST_DEBUG_OBJECT (mix, "comparing mix framerate %d/%d to mixpad's %d/%d",
-      mix->fps_n, mix->fps_d, mixpad->fps_n, mixpad->fps_d);
-  if ((!mix->fps_n && !mix->fps_d) ||
-      ((gint64) mix->fps_n * mixpad->fps_d <
-          (gint64) mixpad->fps_n * mix->fps_d)) {
-    mix->fps_n = mixpad->fps_n;
-    mix->fps_d = mixpad->fps_d;
-    GST_DEBUG_OBJECT (mixpad, "becomes the master pad");
-    mix->master = mixpad;
-  }
+  gst_videomixer_set_master_geometry (mix);
+  GST_VIDEO_MIXER_STATE_UNLOCK (mix);
 
   ret = TRUE;
 
@@ -459,6 +498,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
 static void gst_videomixer_base_init (gpointer g_class);
 static void gst_videomixer_class_init (GstVideoMixerClass * klass);
 static void gst_videomixer_init (GstVideoMixer * videomixer);
+static void gst_videomixer_finalize (GObject * object);
 
 static GstCaps *gst_videomixer_getcaps (GstPad * pad);
 
@@ -466,6 +506,7 @@ static GstFlowReturn gst_videomixer_collected (GstCollectPads * pads,
     GstVideoMixer * mix);
 static GstPad *gst_videomixer_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
+static void gst_videomixer_release_pad (GstElement * element, GstPad * pad);
 static void gst_videomixer_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_videomixer_get_property (GObject * object, guint prop_id,
@@ -526,6 +567,8 @@ gst_videomixer_class_init (GstVideoMixerClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_videomixer_finalize);
+
   gobject_class->get_property = gst_videomixer_get_property;
   gobject_class->set_property = gst_videomixer_set_property;
 
@@ -534,8 +577,43 @@ gst_videomixer_class_init (GstVideoMixerClass * klass)
           GST_TYPE_VIDEO_MIXER_BACKGROUND,
           DEFAULT_BACKGROUND, G_PARAM_READWRITE));
 
-  gstelement_class->request_new_pad = gst_videomixer_request_new_pad;
-  gstelement_class->change_state = gst_videomixer_change_state;
+  gstelement_class->request_new_pad =
+      GST_DEBUG_FUNCPTR (gst_videomixer_request_new_pad);
+  gstelement_class->release_pad =
+      GST_DEBUG_FUNCPTR (gst_videomixer_release_pad);
+  gstelement_class->change_state =
+      GST_DEBUG_FUNCPTR (gst_videomixer_change_state);
+}
+
+static void
+gst_videomixer_collect_free (GstVideoMixerCollect * mixcol)
+{
+  if (mixcol->buffer) {
+    gst_buffer_unref (mixcol->buffer);
+    mixcol->buffer = NULL;
+  }
+}
+
+static void
+gst_videomixer_reset (GstVideoMixer * mix)
+{
+  GSList *walk;
+
+  mix->in_width = 0;
+  mix->in_height = 0;
+  mix->out_width = 0;
+  mix->out_height = 0;
+  mix->fps_n = mix->fps_d = 0;
+  mix->setcaps = FALSE;
+
+  /* clean up collect data */
+  walk = mix->collect->data;
+  while (walk) {
+    GstVideoMixerCollect *data = (GstVideoMixerCollect *) walk->data;
+
+    gst_videomixer_collect_free (data);
+    walk = g_slist_next (walk);
+  }
 }
 
 static void
@@ -546,20 +624,32 @@ gst_videomixer_init (GstVideoMixer * mix)
   mix->srcpad =
       gst_pad_new_from_template (gst_element_class_get_pad_template (klass,
           "src"), "src");
-  gst_pad_set_getcaps_function (GST_PAD (mix->srcpad), gst_videomixer_getcaps);
+  gst_pad_set_getcaps_function (GST_PAD (mix->srcpad),
+      GST_DEBUG_FUNCPTR (gst_videomixer_getcaps));
   gst_element_add_pad (GST_ELEMENT (mix), mix->srcpad);
 
   mix->collect = gst_collect_pads_new ();
   mix->background = DEFAULT_BACKGROUND;
-  mix->in_width = 0;
-  mix->in_height = 0;
-  mix->out_width = 0;
-  mix->out_height = 0;
-  mix->fps_n = mix->fps_d = 0;
 
   gst_collect_pads_set_function (mix->collect,
       (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (gst_videomixer_collected),
       mix);
+
+  mix->state_lock = g_mutex_new ();
+  /* initialize variables */
+  gst_videomixer_reset (mix);
+
+}
+
+static void
+gst_videomixer_finalize (GObject * object)
+{
+  GstVideoMixer *mix = GST_VIDEO_MIXER (object);
+
+  gst_object_unref (mix->collect);
+  g_mutex_free (mix->state_lock);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -627,6 +717,7 @@ gst_videomixer_request_new_pad (GstElement * element,
         templ->direction, "template", templ, NULL);
     g_free (name);
 
+    GST_VIDEO_MIXER_STATE_LOCK (mix);
     mixpad->zorder = mix->numpads;
     mixpad->xpos = DEFAULT_PAD_XPOS;
     mixpad->ypos = DEFAULT_PAD_YPOS;
@@ -636,24 +727,55 @@ gst_videomixer_request_new_pad (GstElement * element,
         gst_collect_pads_add_pad (mix->collect, GST_PAD (mixpad),
         sizeof (GstVideoMixerCollect));
 
-    /* Keep track of eachother */
+    /* Keep track of each other */
     mixcol->mixpad = mixpad;
     mixpad->mixcol = mixcol;
 
     /* Keep an internal list of mixpads for zordering */
     mix->sinkpads = g_slist_append (mix->sinkpads, mixpad);
     mix->numpads++;
+    GST_VIDEO_MIXER_STATE_UNLOCK (mix);
   } else {
     g_warning ("videomixer: this is not our template!\n");
     return NULL;
   }
 
-  /* dd the pad to the element */
+  /* add the pad to the element */
   gst_element_add_pad (element, GST_PAD (mixpad));
 
   return GST_PAD (mixpad);
 }
 
+
+static void
+gst_videomixer_release_pad (GstElement * element, GstPad * pad)
+{
+  GstVideoMixer *mix = NULL;
+  GstVideoMixerPad *mixpad;
+
+  mix = GST_VIDEO_MIXER (element);
+  GST_VIDEO_MIXER_STATE_LOCK (mix);
+  if (G_UNLIKELY (g_slist_find (mix->sinkpads, pad) == NULL)) {
+    g_warning ("Unknown pad %s", GST_PAD_NAME (pad));
+    goto error;
+  }
+
+  mixpad = GST_VIDEO_MIXER_PAD (pad);
+
+  mix->sinkpads = g_slist_remove (mix->sinkpads, pad);
+  gst_videomixer_collect_free (mixpad->mixcol);
+  gst_collect_pads_remove_pad (mix->collect, pad);
+  /* determine possibly new geometry and master */
+  gst_videomixer_set_master_geometry (mix);
+  mix->numpads--;
+  GST_VIDEO_MIXER_STATE_UNLOCK (mix);
+
+  gst_element_remove_pad (element, pad);
+  return;
+error:
+  GST_VIDEO_MIXER_STATE_UNLOCK (mix);
+}
+
 #define BLEND_NORMAL(Y1,U1,V1,Y2,U2,V2,alpha,Y,U,V)     \
         Y = ((Y1*(255-alpha))+(Y2*alpha))>>8;           \
         U = ((U1*(255-alpha))+(U2*alpha))>>8;           \
@@ -1035,7 +1157,7 @@ gst_videomixer_update_queues (GstVideoMixer * mix)
     if (mixcol->buffer != NULL && GST_CLOCK_TIME_IS_VALID (pad->queued)) {
       pad->queued -= interval;
       GST_DEBUG_OBJECT (pad, "queued now %lld", pad->queued);
-      if (pad->queued == 0) {
+      if (pad->queued <= 0) {
         GST_DEBUG ("unreffing buffer");
         gst_buffer_unref (mixcol->buffer);
         mixcol->buffer = NULL;
@@ -1044,76 +1166,6 @@ gst_videomixer_update_queues (GstVideoMixer * mix)
   }
 }
 
-/*
- * The basic idea is to get a buffer on all pads and mix them together.
- * Based on the framerate, buffers are removed from the queues to make room
- * for a new buffer. 
- 
-static void
-gst_videomixer_loop (GstElement * element)
-{
-  GstVideoMixer *mix;
-  GstBuffer *outbuf;
-  gint outsize;
-  gint new_width, new_height;
-  gboolean eos;
-
-  mix = GST_VIDEO_MIXER (element);
-
-  eos = gst_videomixer_fill_queues (mix);
-  if (eos) {
-    gst_pad_push (mix->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
-    gst_element_set_eos (GST_ELEMENT (mix));
-    return;
-  }
-
-  new_width = mix->in_width;
-  new_height = mix->in_height;
-
-  if (new_width != mix->out_width ||
-      new_height != mix->out_height || !GST_PAD_CAPS (mix->srcpad)) {
-    GstCaps *newcaps;
-
-    newcaps = gst_caps_make_writable (
-        gst_pad_get_negotiated_caps (GST_PAD (mix->master)));
-    gst_caps_set_simple (newcaps, "format", GST_TYPE_FOURCC,
-        GST_STR_FOURCC ("AYUV"), "width", G_TYPE_INT, new_width, "height",
-        G_TYPE_INT, new_height, NULL);
-
-    if (GST_PAD_LINK_FAILED (gst_pad_try_set_caps (mix->srcpad, newcaps))) {
-      GST_ELEMENT_ERROR (mix, CORE, NEGOTIATION, (NULL), (NULL));
-      return;
-    }
-
-    mix->out_width = new_width;
-    mix->out_height = new_height;
-  }
-
-  outsize = ROUND_UP_2 (mix->out_width) * ROUND_UP_2 (mix->out_height) * 4;
-
-  outbuf = gst_pad_alloc_buffer_and_set_caps (mix->srcpad, GST_BUFFER_OFFSET_NONE, outsize);
-  switch (mix->background) {
-    case VIDEO_MIXER_BACKGROUND_CHECKER:
-      gst_videomixer_fill_checker (GST_BUFFER_DATA (outbuf),
-          new_width, new_height);
-      break;
-    case VIDEO_MIXER_BACKGROUND_BLACK:
-      gst_videomixer_fill_color (GST_BUFFER_DATA (outbuf),
-          new_width, new_height, 16, 128, 128);
-      break;
-    case VIDEO_MIXER_BACKGROUND_WHITE:
-      gst_videomixer_fill_color (GST_BUFFER_DATA (outbuf),
-          new_width, new_height, 240, 128, 128);
-      break;
-  }
-
-  gst_videomixer_blend_buffers (mix, outbuf);
-
-  gst_videomixer_update_queues (mix);
-
-  gst_pad_push (mix->srcpad, GST_DATA (outbuf));
-}*/
-
 static GstFlowReturn
 gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
 {
@@ -1125,6 +1177,7 @@ gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
   g_return_val_if_fail (GST_IS_VIDEO_MIXER (mix), GST_FLOW_ERROR);
 
   GST_LOG ("all pads are collected");
+  GST_VIDEO_MIXER_STATE_LOCK (mix);
 
   eos = gst_videomixer_fill_queues (mix);
 
@@ -1133,14 +1186,15 @@ gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
     GST_LOG ("all our sinkpads are EOS, pushing downstream");
     gst_pad_push_event (mix->srcpad, gst_event_new_eos ());
     ret = GST_FLOW_WRONG_STATE;
-    goto beach;
+    goto error;
   }
 
   /* Calculating out buffer size from input size */
   outsize = ROUND_UP_2 (mix->in_width) * ROUND_UP_2 (mix->in_height) * 4;
 
   /* If geometry has changed we need to set new caps on the buffer */
-  if (mix->in_width != mix->out_width || mix->in_height != mix->out_height) {
+  if (mix->in_width != mix->out_width || mix->in_height != mix->out_height
+      || mix->setcaps) {
     GstCaps *newcaps = NULL;
 
     newcaps = gst_caps_make_writable
@@ -1152,6 +1206,7 @@ gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
 
     mix->out_width = mix->in_width;
     mix->out_height = mix->in_height;
+    mix->setcaps = FALSE;
 
     ret =
         gst_pad_alloc_buffer_and_set_caps (mix->srcpad, GST_BUFFER_OFFSET_NONE,
@@ -1164,7 +1219,7 @@ gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
   }
 
   if (ret != GST_FLOW_OK) {
-    goto beach;
+    goto error;
   }
 
   switch (mix->background) {
@@ -1185,11 +1240,19 @@ gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
   gst_videomixer_blend_buffers (mix, outbuf);
 
   gst_videomixer_update_queues (mix);
+  GST_VIDEO_MIXER_STATE_UNLOCK (mix);
 
   ret = gst_pad_push (mix->srcpad, outbuf);
 
 beach:
   return ret;
+
+  /* ERRORS */
+error:
+  {
+    GST_VIDEO_MIXER_STATE_UNLOCK (mix);
+    goto beach;
+  }
 }
 
 static void
@@ -1249,6 +1312,14 @@ gst_videomixer_change_state (GstElement * element, GstStateChange transition)
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_videomixer_reset (mix);
+      break;
+    default:
+      break;
+  }
+
   return ret;
 }
 
