@@ -60,11 +60,6 @@
 #include "gst/riff/riff-media.h"
 #include <gst/gst-i18n-plugin.h>
 
-/* FIXME: remove again when we depend on -base >= 0.10.9.1 */
-#ifndef GST_RIFF_TAG_fact
-#define GST_RIFF_TAG_fact GST_MAKE_FOURCC ('f','a','c','t')
-#endif
-
 GST_DEBUG_CATEGORY_STATIC (wavparse_debug);
 #define GST_CAT_DEFAULT (wavparse_debug)
 
@@ -121,6 +116,7 @@ static GstStaticPadTemplate src_template_factory =
         "depth = (int) { 8, 16, 24, 32 }, "
         "rate = (int) [ 8000, 96000 ], "
         "channels = (int) [ 1, 8 ]; "
+        "audio/ms-gsm; "
         "audio/mpeg, "
         "mpegversion = (int) 1, "
         "layer = (int) [ 1, 3 ], "
@@ -809,6 +805,8 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
   } else {
     GST_DEBUG_OBJECT (wav, "doing seek without event");
     flags = 0;
+    cur_type = GST_SEEK_TYPE_SET;
+    stop_type = GST_SEEK_TYPE_SET;
   }
 
   flush = flags & GST_SEEK_FLAG_FLUSH;
@@ -836,8 +834,7 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
     stop = seeksegment.duration;
 
   GST_DEBUG_OBJECT (wav, "cur_type =%d", cur_type);
-  if ((cur_type != GST_SEEK_TYPE_NONE) &&
-      (seeksegment.last_stop != GST_CLOCK_TIME_NONE)) {
+  if ((cur_type != GST_SEEK_TYPE_NONE)) {
     wav->offset =
         gst_util_uint64_scale_int (seeksegment.last_stop, wav->bps, GST_SECOND);
     GST_LOG_OBJECT (wav, "offset=%" G_GUINT64_FORMAT, wav->offset);
@@ -846,12 +843,11 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
     wav->offset += wav->datastart;
     GST_LOG_OBJECT (wav, "offset=%" G_GUINT64_FORMAT, wav->offset);
   } else {
-    GST_DEBUG_OBJECT (wav, "last_stop == -1");
-    wav->offset = wav->datastart;
-    GST_LOG_OBJECT (wav, "offset=%" G_GUINT64_FORMAT, wav->offset);
+    GST_LOG_OBJECT (wav, "continue from offset=%" G_GUINT64_FORMAT,
+        wav->offset);
   }
 
-  if (stop != GST_CLOCK_TIME_NONE) {
+  if (stop_type != GST_SEEK_TYPE_NONE) {
     wav->end_offset = gst_util_uint64_scale_int (stop, wav->bps, GST_SECOND);
     GST_LOG_OBJECT (wav, "end_offset=%" G_GUINT64_FORMAT, wav->end_offset);
     wav->end_offset -= (wav->end_offset % wav->bytes_per_sample);
@@ -859,17 +855,15 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
     wav->end_offset += wav->datastart;
     GST_LOG_OBJECT (wav, "end_offset=%" G_GUINT64_FORMAT, wav->end_offset);
   } else {
-    GST_DEBUG_OBJECT (wav, "stop == -1");
-    wav->end_offset = wav->datasize + wav->datastart;
-    GST_LOG_OBJECT (wav, "end_offset=%" G_GUINT64_FORMAT, wav->end_offset);
+    GST_LOG_OBJECT (wav, "continue to end_offset=%" G_GUINT64_FORMAT,
+        wav->end_offset);
   }
 
   /* make sure filesize is not exceeded due to rounding errors or so,
    * same precaution as in _stream_headers */
   bformat = GST_FORMAT_BYTES;
-  if (gst_pad_query_peer_duration (wav->sinkpad, &bformat, &upstream_size)) {
+  if (gst_pad_query_peer_duration (wav->sinkpad, &bformat, &upstream_size))
     wav->end_offset = MIN (wav->end_offset, upstream_size);
-  }
 
   wav->offset = MIN (wav->offset, wav->end_offset);
   wav->dataleft = wav->end_offset - wav->offset;
@@ -888,12 +882,14 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
       /* we are running the current segment and doing a non-flushing seek,
        * close the segment first based on the last_stop. */
       GST_DEBUG_OBJECT (wav, "closing running segment %" G_GINT64_FORMAT
-          " to %" G_GINT64_FORMAT, wav->segment.start, wav->segment.last_stop);
+          " to %" G_GINT64_FORMAT, wav->segment.accum, wav->segment.last_stop);
 
       gst_pad_push_event (wav->srcpad,
           gst_event_new_new_segment (TRUE,
               wav->segment.rate, wav->segment.format,
-              wav->segment.start, wav->segment.last_stop, wav->segment.time));
+              wav->segment.accum, wav->segment.last_stop, wav->segment.accum));
+
+      seeksegment.accum = wav->segment.last_stop;
     }
   }
 
@@ -907,14 +903,15 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
 
   /* now send the newsegment */
   GST_DEBUG_OBJECT (wav, "Sending newsegment from %" G_GINT64_FORMAT
-      " to %" G_GINT64_FORMAT, wav->segment.start, stop);
+      " to %" G_GINT64_FORMAT, wav->segment.last_stop, stop);
 
-	/* store the newsegment event so it can be sent from the streaming thread. */
-	if (wav->newsegment)
-		gst_event_unref (wav->newsegment);
+  /* store the newsegment event so it can be sent from the streaming thread. */
+  if (wav->newsegment)
+    gst_event_unref (wav->newsegment);
   wav->newsegment =
       gst_event_new_new_segment (FALSE, wav->segment.rate,
-      wav->segment.format, wav->segment.last_stop, stop, wav->segment.time);
+      wav->segment.format, wav->segment.last_stop, stop,
+      wav->segment.last_stop);
 
   wav->segment_running = TRUE;
   if (!wav->streaming) {
@@ -1182,7 +1179,7 @@ gst_wavparse_stream_headers (GstWavParse * wav)
           /* We will continue parsing tags 'till end */
           wav->offset += size;
         }
-        GST_DEBUG_OBJECT (wav, "datasize = %ld", size);
+        GST_DEBUG_OBJECT (wav, "datasize = %d", size);
         break;
       }
       case GST_RIFF_TAG_fact:{
@@ -1236,7 +1233,7 @@ gst_wavparse_stream_headers (GstWavParse * wav)
     wav->bps =
         (guint32) gst_util_uint64_scale ((guint64) wav->rate, wav->datasize,
         (guint64) wav->fact);
-    GST_DEBUG_OBJECT (wav, "calculated bps : %ld", wav->bps);
+    GST_DEBUG_OBJECT (wav, "calculated bps : %d", wav->bps);
   }
   if (wav->bps <= 0)
     goto no_bitrate;
@@ -1445,7 +1442,8 @@ iterate_adapter:
    * amounts of data regardless of the playback rate */
   desired =
       MIN (gst_guint64_to_gdouble (wav->dataleft),
-      MAX_BUFFER_SIZE * ABS (wav->segment.rate));
+      MAX_BUFFER_SIZE * wav->segment.abs_rate);
+
   if (desired >= wav->blockalign && wav->blockalign > 0)
     desired -= (desired % wav->blockalign);
 
@@ -1476,11 +1474,11 @@ iterate_adapter:
     gst_wavparse_add_src_pad (wav, buf);
   }
 
-	/* If we have a pending newsegment send it now. */
-	if (G_UNLIKELY (wav->newsegment != NULL)) {
-		gst_pad_push_event (wav->srcpad, wav->newsegment);
-		wav->newsegment = NULL;
-	}	
+  /* If we have a pending newsegment send it now. */
+  if (G_UNLIKELY (wav->newsegment != NULL)) {
+    gst_pad_push_event (wav->srcpad, wav->newsegment);
+    wav->newsegment = NULL;
+  }
 
   obtained = GST_BUFFER_SIZE (buf);
 
@@ -1963,10 +1961,6 @@ gst_wavparse_change_state (GstElement * element, GstStateChange transition)
   GstStateChangeReturn ret;
   GstWavParse *wav = GST_WAVPARSE (element);
 
-  GST_DEBUG_OBJECT (wav, "changing state %s - %s",
-      gst_element_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
-      gst_element_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
-
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
